#include "TelnetApp.h"
#include "TelnetWindow.h"
#include "TelnetOpenWindow.h"
#include "TelnetMessages.h"
#include "TelnetErrors.h"

#include <app/Application.h>
#include <app/Message.h>
#include <interface/Alert.h>
#include <interface/MenuItem.h>
#include <kernel/fs_attr.h>
#include <storage/Node.h>

#include <syslog.h>

bool
TelnetApp::AddWindow(const char *host, const char *port, BRect frame, entry_ref *ref)
{
	TelnetWindow *new_win;
	if (!(new_win = new TelnetWindow(frame, host, port)))
		return FALSE;
	if (ref)
		new_win->SetSaveFile(ref);

	num_windows++;
	return TRUE;
}

bool
TelnetApp::AddWindow(const char *host, const char *port)
{
	return AddWindow(host, port, BRect(32*(num_windows+1), 32*(num_windows+1),
									   632+32*num_windows, 432+32*num_windows));
}

bool
TelnetApp::AddWindow(entry_ref *ref)
{
		BAlert *alert;
		char *host, *port;
		char type[B_FILE_NAME_LENGTH];
		BFile file(ref, B_READ_ONLY);
		BNodeInfo *node;
		BRect frame;
		BResources res;
		attr_info info;
		status_t rc;

		if (file.InitCheck() != B_NO_ERROR) {
			alert = new BAlert("", "Couldn't open file.", ERROR_BUTTON, NULL, NULL,
							   B_WIDTH_AS_USUAL, B_STOP_ALERT);
			alert->Go();
			delete alert;
			return false;
		}
		node = new BNodeInfo(&file);
		node->GetType(type);
		delete node;
		if (strcmp(type, FILE_TYPE)) {
			alert = new BAlert("", "File is not a Telnet profile.", ERROR_BUTTON,
							   NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
			alert->Go();
			delete alert;
			return false;
		}

		rc = file.GetAttrInfo("host", &info);
		if (rc != B_NO_ERROR) {
			alert = new BAlert("", "Couldn't get attributes from profile.", ERROR_BUTTON,
							   NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
			alert->Go();
			delete alert;
			return false;
		}
		host = new char[info.size];
		file.ReadAttr("host", B_STRING_TYPE, 0, host, info.size);

		rc = file.GetAttrInfo("port", &info);
		if (rc == B_NO_ERROR) {
			port = new char[info.size];
			file.ReadAttr("port", B_STRING_TYPE, 0, port, info.size);
		} else {
			port = new char[strlen("telnet")+1];
			strcpy(port, "telnet");
		}

		rc = res.SetTo(&file);
		if (rc != B_NO_ERROR)
			rc = AddWindow(host, port);
		else {
			rc = res.ReadResource(B_RECT_TYPE, 0, &frame, 0, sizeof(frame));
			if (rc == B_NO_ERROR) {
				rc = AddWindow(host, port, frame, ref);
			} else
				rc = AddWindow(host, port);
		}
		delete host; delete port;
		return rc;
}

TelnetApp::TelnetApp()
		  :BApplication("application/x-vnd.kublai-Telnet")
{
	num_windows = 0;
	opening = false;
}

void TelnetApp::AboutRequested()
{
	BAlert *about_box = new BAlert("", ABOUT_MESS, "OK", NULL, NULL, B_WIDTH_AS_USUAL,
								    B_INFO_ALERT);

	if (about_box)
		about_box->Go();
}

void
TelnetApp::ArgvReceived(int32 argc, char **argv)
{
	BMessage msg(OPEN_CONNECT_MSG);
	char *p, *q;

	if (argc > 3) {
		fprintf(stderr, "Usage: %s hostname [port].\n", argv[0]);
		PostMessage(B_QUIT_REQUESTED);
		return;
	} else if (argc == 3) {
		// We have a host and a port.
		msg.AddString("port", argv[2]);
		msg.AddString("host", argv[1]);
	} else if (argc == 2) {
		if (strncmp(argv[1], "telnet:", sizeof("telnet:")-1) == 0) {
			// We have a URL, so parse that way.
			p = index(argv[1], ':');
			q = index(p, '@');							// In case we have user@host
			if (q)
				p = q;
			p++;

			if (!*p)
				return;

			// Check for a possible port.
			q = index(p, ':');
			if (q && *(q+1)) {
				msg.AddString("port", q+1);
				*q = '\0';
			} else
				msg.AddString("port", "telnet");
			msg.AddString("host", p);
		} else {
			// No port information.
			msg.AddString("port", "telnet");
			msg.AddString("host", argv[1]);
		}
	}
	opening = true;
	PostMessage(&msg);
}

void
TelnetApp::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
	case NEW_MSG:
		// Open up the open dialog box.
		TelnetOpenWindow *open_window = new TelnetOpenWindow(BPoint(50, 50));
		if (open_window)
			num_windows++;
		break;
	case PREF_MSG: {
		// Open up the prefs dialog box.
	}; break;
	case WIN_CLOSE_MSG: {
		if (!--num_windows)
			PostMessage(B_QUIT_REQUESTED);
	}; break;
	case OPEN_CONNECT_MSG:
		if (!AddWindow(msg->FindString("host"), msg->FindString("port"))) {
			syslog(LOG_ERR, "Couldn't create new window.");
			return;
		}
		if (!num_windows)
			PostMessage(B_QUIT_REQUESTED);
		break;
	case ARP_ABOUT_MSG: {
		BAlert *about_box  = new BAlert("", ABOUT_ARP_MESS, "OK", NULL, NULL,
										 B_WIDTH_AS_USUAL, B_INFO_ALERT);
		if (about_box)
			about_box->Go();
	}; break;
	case B_REFS_RECEIVED:
		RefsReceived(msg);
		break;
	default:
		BApplication::MessageReceived(msg);
	}
}


void
TelnetApp::ReadyToRun()
{
	// First things first, make sure the computer is in a state that can
	// run a program such as Telnet.
	if (!is_computer_on()) {
		syslog(LOG_CRIT, "Computer is off.");
		PostMessage(B_QUIT_REQUESTED);
	}
/*
 *	double fire_quotient = is_computer_on_fire();
 *	if (fire_quotient) {
 *		fprintf(stderr, "Error: Computer is on fire: %f.\n", fire_quotient);
 *		PostMessage(B_QUIT_REQUESTED);
 *	}
 */

	// Now open up a dialog box asking for the host to connect to.
	if (!num_windows && !opening)
		PostMessage(NEW_MSG);
}

void
TelnetApp::RefsReceived(BMessage *msg)
{
	BAlert *alert;
	entry_ref ref;
	int i = 0;
	status_t rc;

	while (msg->HasRef("refs", i)) {
		rc = msg->FindRef("refs", i++, &ref);
		if (rc != B_NO_ERROR) {
			alert = new BAlert("", "No files to open.", ERROR_BUTTON, NULL, NULL,
							   B_WIDTH_AS_USUAL, B_STOP_ALERT);
			alert->Go();
			delete alert;
			continue;
		}

		if (AddWindow(&ref))
			opening = true;
	}
}